import 'package:flutter/material.dart';

class RotationButton extends StatefulWidget {
  final Function(double, RotationState) rotationProgress;
  final String imgAsset;
  final String imgAssetBack;

  RotationButton(
      {@required this.rotationProgress,
        this.imgAsset = "packages/rdkr_widgets/images/rotation_arrow.png",
        this.imgAssetBack = "packages/rdkr_widgets/images/rotation_arrow_back.png"});

  @override
  _RotationButtonState createState() => _RotationButtonState();
}

class _RotationButtonState extends State<RotationButton>
    with SingleTickerProviderStateMixin {
  final double rotationEnd = 320;
  RotationState state = RotationState.FRONT;
  double rotation = 0;
  Animation animation;
  AnimationController animationController;
  Widget arrows;

  @override
  void initState() {
    super.initState();
    switchArrows();
    animationController =
        AnimationController(duration: Duration(milliseconds: 500), vsync: this);
    animation = Tween<double>(begin: 0, end: rotationEnd).animate(
        CurvedAnimation(parent: animationController, curve: Curves.easeInOut));
    animationController.addListener(() {
      setState(() {
        rotation = animation.value;
        if (rotation > 0 && rotation < rotationEnd) {
          state = RotationState.PROGRESS;
        }
        callListener();
      });
    });
    animationController.addStatusListener((status) {
      print("animation $status");
      print("state = $state");
      if (status == AnimationStatus.completed ||
          status == AnimationStatus.dismissed) {
        if (rotation == 0) {
          state = RotationState.FRONT;
        } else if (rotation == rotationEnd) {
          state = RotationState.BACK;
        }
        switchArrows();
        callListener();
      } else if (status == AnimationStatus.forward ||
          status == AnimationStatus.reverse) {
        state = RotationState.PROGRESS;
      }
      setState(() {});
    });
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: toggle,
      onPanUpdate: (details) {
        state = RotationState.DRAGGING;
        setState(() {
          rotation -= details.delta.dx.toInt() * 2;
          if (rotation < 0) {
            rotation = 0;
          }
          if (rotation > rotationEnd) {
            rotation = rotationEnd;
          }
        });
        callListener();
      },
      onPanEnd: (details) {
        state = RotationState.PROGRESS;
        if (rotation > rotationEnd / 2) {
          animationController.forward(from: rotation / rotationEnd);
        } else {
          animationController.reverse(from: rotation / rotationEnd);
        }
      },
      child: Transform(
        transform: Matrix4.identity()
          ..setEntry(3, 2, 0.001) // perspective
          ..rotateX(250)
          ..rotateZ(0.01 * rotation), // changed
        alignment: FractionalOffset.center,
        child: Container(
          width: 150,
          height: 150,
          child: AnimatedSwitcher(
            duration: Duration(milliseconds: 200),
            switchInCurve: Curves.easeInOut,
            switchOutCurve: Curves.easeInOut,
//            child: Image(image: AssetImage(widget.imgAsset)),
            child: arrows,
          ),
        ),
      ),
    );
  }

  void switchArrows() {
    if (state == RotationState.FRONT) {
      setState(() {
        arrows = Image(image: AssetImage(widget.imgAsset));
      });
    } else if (state == RotationState.BACK) {
      setState(() {
        arrows = Image(image: AssetImage(widget.imgAssetBack));
      });
    }
  }

  void toggle() {
    print("toggle: state = $state");
    if (state == RotationState.FRONT) {
      animationController.forward();
    } else if (state == RotationState.BACK) {
      animationController.reverse();
    } else if (state == RotationState.PROGRESS) {
      if (rotation > rotationEnd / 2) {
        animationController.forward(from: rotation / rotationEnd);
      } else {
        animationController.reverse(from: rotation / rotationEnd);
      }
    }
  }

  void callListener() {
    if (widget.rotationProgress != null) {
      widget.rotationProgress(rotation / rotationEnd, state);
    }
  }
}

enum RotationState {
  DRAGGING,
  PROGRESS,
  FRONT,
  BACK,
}
